/*-
 * Copyright (c) 2000 Mitsuru IWASAKI <iwasaki@FreeBSD.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *	$FreeBSD$
 */

#include <sys/param.h>
#include <sys/mman.h>
#include <sys/stat.h>

#include <assert.h>
#include <err.h>
#include <fcntl.h>
#include <stdio.h>
#include <unistd.h>

#include <acpi.h>

ACPI_TABLE_HEADER	dummy_dsdt_table = {
	"DSDT", 123, 1, 123, "OEMID", "OEMTBLID", 1, "CRID", 1
};

static int
load_dsdt(const char *dsdtfile)
{
	u_int8_t		*code;
	struct stat		sb;
	int			fd;
	int			error;
	ACPI_TABLE_DESC		TableInfo;

	fd = open(dsdtfile, O_RDONLY, 0);
	if (fd == -1) {
		perror("open");
		exit(-1);
	}
	if (fstat(fd, &sb) == -1) {
		perror("fstat");
		exit(-1);
	}
	if ((code = mmap(NULL, sb.st_size, PROT_READ, MAP_PRIVATE, fd, 0)) == NULL) {
		perror("mmap");
		exit(-1);
	}
	if ((error = AcpiInitializeSubsystem()) != AE_OK) {
		return;
	}

	/*
	 * Microsoft asl.exe generates 0x23 byte additional info.
	 * at the begining of the file, so just ignore it.   
	 */
	if (strncmp(code, "DSDT", 4) == 0) {
		bcopy(code, &dummy_dsdt_table, sizeof(dummy_dsdt_table));
		code += 0x23;
		sb.st_size -= 0x23;
	}

	dummy_dsdt_table.Length = sb.st_size + sizeof(dummy_dsdt_table);

	AcpiGbl_AcpiTables[ACPI_TABLE_DSDT].Pointer = AcpiGbl_DSDT = &dummy_dsdt_table;
	AcpiGbl_AcpiTables[ACPI_TABLE_DSDT].AmlPointer = code;
	AcpiGbl_AcpiTables[ACPI_TABLE_DSDT].AmlLength = sb.st_size;

	AcpiNsLoadNamespace();
	AcpiDbInitialize();
	AcpiGbl_DebuggerConfiguration = 0;
	AcpiDbUserCommands(NULL, NULL);
}

static void
usage(const char *progname)
{

	printf("usage: %s dsdt_file\n", progname);
	exit(1);
}

int
main(int argc, char *argv[])
{
	char	*progname;

	progname = argv[0];

	if (argc == 1) {
		usage(progname);
	}

	AcpiDbgLevel = DEBUG_DEFAULT;
	load_dsdt(argv[1]);

	return (0);
}
